Cldr.Message.V2.Parser (Cldr Messages v2.0.0)

Copy Markdown View Source

Implements a parser for ICU MessageFormat 2.

Summary

Functions

Parses the given binary as message.

Parses a MessageFormat 2 message string.

Parses a MessageFormat 2 message string, raising on error.

Functions

message(binary, opts \\ [])

@spec message(binary(), keyword()) ::
  {:ok, [term()], rest, context, line, byte_offset}
  | {:error, reason, rest, context, line, byte_offset}
when line: {pos_integer(), byte_offset},
     byte_offset: non_neg_integer(),
     rest: binary(),
     reason: String.t(),
     context: map()

Parses the given binary as message.

Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the message (start position) as {line, offset_to_start_of_line}.

To column where the error occurred can be inferred from byte_offset - offset_to_start_of_line.

Options

  • :byte_offset - the byte offset for the whole binary, defaults to 0
  • :line - the line and the byte offset into that line, defaults to {1, byte_offset}
  • :context - the initial context value. It will be converted to a map

parse(input)

@spec parse(String.t()) :: {:ok, list()} | {:error, String.t()}

Parses a MessageFormat 2 message string.

Arguments

  • input is a MF2 message string.

Returns

  • {:ok, ast} where ast is the parsed message AST, or

  • {:error, reason} if the message cannot be parsed.

parse!(input)

@spec parse!(String.t()) :: list() | no_return()

Parses a MessageFormat 2 message string, raising on error.

Same as parse/1 but returns the AST directly or raises ArgumentError.